import React, { PureComponent, useState, useRef, useEffect, Component } from 'react';
import { Modifier, SelectionState, EditorState, AtomicBlockUtils, RichUtils, KeyBindingUtil, DefaultDraftBlockRenderMap, getDefaultKeyBinding, getVisibleSelectionRect } from 'draft-js';
import { filterEditorState, condenseBlocks } from 'draftjs-filters';
import Editor from 'draft-js-plugins-editor';
import { blockDepthStyleFn, getListNestingStyles, createEditorStateFromRaw, registerCopySource, serialiseEditorStateToRaw, handleDraftEditorPastedText } from 'draftjs-conductor';
export { createEditorStateFromRaw, serialiseEditorStateToRaw } from 'draftjs-conductor';
import decorateComponentWithProps from 'decorate-component-with-props';
import isSoftNewlineEvent from 'draft-js/lib/isSoftNewlineEvent';
import Tippy from '@tippyjs/react';
import { useCombobox } from 'downshift';
import DraftEditorBlock from 'draft-js/lib/DraftEditorBlock.react';

// See https://github.com/facebook/draft-js/blob/master/src/model/immutable/DefaultDraftBlockRenderMap.js
const BLOCK_TYPE = {
    // This is used to represent a normal text block (paragraph).
    UNSTYLED: "unstyled",
    HEADER_ONE: "header-one",
    HEADER_TWO: "header-two",
    HEADER_THREE: "header-three",
    HEADER_FOUR: "header-four",
    HEADER_FIVE: "header-five",
    HEADER_SIX: "header-six",
    UNORDERED_LIST_ITEM: "unordered-list-item",
    ORDERED_LIST_ITEM: "ordered-list-item",
    BLOCKQUOTE: "blockquote",
    CODE: "code-block",
    // This represents a "custom" block, not for rich text, with arbitrary content.
    ATOMIC: "atomic",
};
const ENTITY_TYPE = {
    LINK: "LINK",
    IMAGE: "IMAGE",
    HORIZONTAL_RULE: "HORIZONTAL_RULE",
};
// See https://github.com/facebook/draft-js/blob/master/src/model/immutable/DefaultDraftInlineStyle.js
const INLINE_STYLE = {
    BOLD: "BOLD",
    ITALIC: "ITALIC",
    CODE: "CODE",
    UNDERLINE: "UNDERLINE",
    STRIKETHROUGH: "STRIKETHROUGH",
    MARK: "MARK",
    QUOTATION: "QUOTATION",
    SMALL: "SMALL",
    SAMPLE: "SAMPLE",
    INSERT: "INSERT",
    DELETE: "DELETE",
    KEYBOARD: "KEYBOARD",
    SUPERSCRIPT: "SUPERSCRIPT",
    SUBSCRIPT: "SUBSCRIPT",
};
const BLOCK_TYPES = Object.values(BLOCK_TYPE);
const ENTITY_TYPES = Object.values(ENTITY_TYPE);
const INLINE_STYLES = Object.values(INLINE_STYLE);
const KEY_COMMANDS = [
    ...BLOCK_TYPES,
    ...ENTITY_TYPES,
    ...INLINE_STYLES,
    // Lowercase identifiers used by Draft.js
    // See https://github.com/facebook/draft-js/blob/585af35c3a8c31fefb64bc884d4001faa96544d3/src/model/constants/DraftEditorCommand.js#L58-L61.
    "bold",
    "italic",
    "underline",
    "code",
];
const FONT_FAMILY_MONOSPACE = "Consolas, Menlo, Monaco, Lucida Console, Liberation Mono, DejaVu Sans Mono, Bitstream Vera Sans Mono, Courier New, monospace, sans-serif";
// See https://github.com/facebook/draft-js/blob/master/src/model/immutable/DefaultDraftInlineStyle.js
const CUSTOM_STYLE_MAP = {
    [INLINE_STYLE.BOLD]: {
        fontWeight: "bold",
    },
    [INLINE_STYLE.ITALIC]: {
        fontStyle: "italic",
    },
    [INLINE_STYLE.STRIKETHROUGH]: {
        textDecoration: "line-through",
    },
    [INLINE_STYLE.UNDERLINE]: {
        textDecoration: "underline",
    },
    [INLINE_STYLE.CODE]: {
        padding: "0.2em 0.3125em",
        margin: "0",
        fontSize: "85%",
        backgroundColor: "rgba(27, 31, 35, 0.05)",
        fontFamily: FONT_FAMILY_MONOSPACE,
        borderRadius: "3px",
    },
    [INLINE_STYLE.MARK]: {
        backgroundColor: "yellow",
    },
    [INLINE_STYLE.QUOTATION]: {
        fontStyle: "italic",
    },
    [INLINE_STYLE.SMALL]: {
        fontSize: "smaller",
    },
    [INLINE_STYLE.SAMPLE]: {
        fontFamily: FONT_FAMILY_MONOSPACE,
    },
    [INLINE_STYLE.INSERT]: {
        textDecoration: "underline",
    },
    [INLINE_STYLE.DELETE]: {
        textDecoration: "line-through",
    },
    [INLINE_STYLE.KEYBOARD]: {
        fontFamily: FONT_FAMILY_MONOSPACE,
        padding: "3px 5px",
        fontSize: "11px",
        lineHeight: "10px",
        color: "#444d56",
        verticalAlign: "middle",
        backgroundColor: "#fafbfc",
        border: "solid 1px #c6cbd1",
        borderBottomColor: "#959da5",
        borderRadius: "3px",
        boxShadow: "inset 0 -1px 0 #959da5",
    },
    [INLINE_STYLE.SUPERSCRIPT]: {
        fontSize: "80%",
        verticalAlign: "super",
        lineHeight: "1",
    },
    [INLINE_STYLE.SUBSCRIPT]: {
        fontSize: "80%",
        verticalAlign: "sub",
        lineHeight: "1",
    },
};
const BR_TYPE = "BR";
const UNDO_TYPE = "undo";
const REDO_TYPE = "redo";
// Originally from https://github.com/facebook/draft-js/blob/master/src/component/utils/getDefaultKeyBinding.js.
const KEY_CODES = {
    K: 75,
    B: 66,
    U: 85,
    J: 74,
    I: 73,
    X: 88,
    "0": 48,
    "1": 49,
    "2": 50,
    "3": 51,
    "4": 52,
    "5": 53,
    "6": 54,
    "7": 55,
    "8": 56,
    ".": 190,
    ",": 188,
};
const INPUT_BLOCK_MAP = {
    "* ": BLOCK_TYPE.UNORDERED_LIST_ITEM,
    "- ": BLOCK_TYPE.UNORDERED_LIST_ITEM,
    "1. ": BLOCK_TYPE.ORDERED_LIST_ITEM,
    "# ": BLOCK_TYPE.HEADER_ONE,
    "## ": BLOCK_TYPE.HEADER_TWO,
    "### ": BLOCK_TYPE.HEADER_THREE,
    "#### ": BLOCK_TYPE.HEADER_FOUR,
    "##### ": BLOCK_TYPE.HEADER_FIVE,
    "###### ": BLOCK_TYPE.HEADER_SIX,
    "> ": BLOCK_TYPE.BLOCKQUOTE,
    // It makes more sense not to require a space here.
    // This matches how Dropbox Paper operates.
    "```": BLOCK_TYPE.CODE,
};
const INPUT_STYLE_MAP = [
    // Order matters, as shorter patterns are contained in the longer ones.
    { pattern: "**", type: INLINE_STYLE.BOLD },
    { pattern: "__", type: INLINE_STYLE.BOLD },
    { pattern: "*", type: INLINE_STYLE.ITALIC },
    { pattern: "_", type: INLINE_STYLE.ITALIC },
    { pattern: "~~", type: INLINE_STYLE.STRIKETHROUGH },
    { pattern: "~", type: INLINE_STYLE.STRIKETHROUGH },
    { pattern: "`", type: INLINE_STYLE.CODE },
].map(({ pattern, type }) => {
    const pat = pattern.replace(/[.*+?^${}()|[\]\\]/g, "\\$&");
    const char = pattern[0].replace(/[.*+?^${}()|[\]\\]/g, "\\$&");
    // https://regexper.com/#%28%5Cs%7C%5E%29__%28%5B%5E%5Cs_%5D%7B1%2C2%7D%7C%5B%5E%5Cs_%5D.%2B%5B%5E%5Cs_%5D%29__%24
    // This is stored as an escaped string instead of a RegExp object because they are stateful.
    // This regex encapsulates a few rules:
    // - The pattern must be preceded by whitespace, or be at the start of the input.
    // - The pattern must end the input.
    // - In-between the start and end patterns, there can't be only whitespace or characters from the pattern.
    // - There has to be at least 1 char that's not whitespace or the pattern’s char.
    const regex = `(\\s|^)${pat}([^\\s${char}]{1,2}|[^\\s${char}].+[^\\s${char}])${pat}$`;
    return {
        pattern,
        type,
        regex,
    };
});
const INPUT_ENTITY_MAP = {
    [ENTITY_TYPE.HORIZONTAL_RULE]: "---",
};
const LABELS = {
    [BLOCK_TYPE.UNSTYLED]: "P",
    [BLOCK_TYPE.HEADER_ONE]: "H1",
    [BLOCK_TYPE.HEADER_TWO]: "H2",
    [BLOCK_TYPE.HEADER_THREE]: "H3",
    [BLOCK_TYPE.HEADER_FOUR]: "H4",
    [BLOCK_TYPE.HEADER_FIVE]: "H5",
    [BLOCK_TYPE.HEADER_SIX]: "H6",
    [BLOCK_TYPE.UNORDERED_LIST_ITEM]: "UL",
    [BLOCK_TYPE.ORDERED_LIST_ITEM]: "OL",
    [BLOCK_TYPE.CODE]: "{ }",
    [BLOCK_TYPE.BLOCKQUOTE]: "❝",
    [INLINE_STYLE.BOLD]: "𝐁",
    [INLINE_STYLE.ITALIC]: "𝘐",
    [INLINE_STYLE.CODE]: "{ }",
    [INLINE_STYLE.UNDERLINE]: "U",
    [INLINE_STYLE.STRIKETHROUGH]: "S",
    [INLINE_STYLE.MARK]: "☆",
    [INLINE_STYLE.QUOTATION]: "❛",
    [INLINE_STYLE.SMALL]: "Small",
    [INLINE_STYLE.SAMPLE]: "Data",
    [INLINE_STYLE.INSERT]: "Ins",
    [INLINE_STYLE.DELETE]: "Del",
    [INLINE_STYLE.SUPERSCRIPT]: "Sup",
    [INLINE_STYLE.SUBSCRIPT]: "Sub",
    [INLINE_STYLE.KEYBOARD]: "⌘",
    [ENTITY_TYPE.LINK]: "🔗",
    [ENTITY_TYPE.IMAGE]: "🖼",
    [ENTITY_TYPE.HORIZONTAL_RULE]: "―",
    [BR_TYPE]: "↵",
    [UNDO_TYPE]: "↺",
    [REDO_TYPE]: "↻",
};
const DESCRIPTIONS = {
    [BLOCK_TYPE.UNSTYLED]: "Paragraph",
    [BLOCK_TYPE.HEADER_ONE]: "Heading 1",
    [BLOCK_TYPE.HEADER_TWO]: "Heading 2",
    [BLOCK_TYPE.HEADER_THREE]: "Heading 3",
    [BLOCK_TYPE.HEADER_FOUR]: "Heading 4",
    [BLOCK_TYPE.HEADER_FIVE]: "Heading 5",
    [BLOCK_TYPE.HEADER_SIX]: "Heading 6",
    [BLOCK_TYPE.UNORDERED_LIST_ITEM]: "Bulleted list",
    [BLOCK_TYPE.ORDERED_LIST_ITEM]: "Numbered list",
    [BLOCK_TYPE.BLOCKQUOTE]: "Blockquote",
    [BLOCK_TYPE.CODE]: "Code block",
    [INLINE_STYLE.BOLD]: "Bold",
    [INLINE_STYLE.ITALIC]: "Italic",
    [INLINE_STYLE.CODE]: "Code",
    [INLINE_STYLE.UNDERLINE]: "Underline",
    [INLINE_STYLE.STRIKETHROUGH]: "Strikethrough",
    [INLINE_STYLE.MARK]: "Highlight",
    [INLINE_STYLE.QUOTATION]: "Inline quotation",
    [INLINE_STYLE.SMALL]: "Small",
    [INLINE_STYLE.SAMPLE]: "Program output",
    [INLINE_STYLE.INSERT]: "Inserted",
    [INLINE_STYLE.DELETE]: "Deleted",
    [INLINE_STYLE.KEYBOARD]: "Shortcut key",
    [INLINE_STYLE.SUPERSCRIPT]: "Superscript",
    [INLINE_STYLE.SUBSCRIPT]: "Subscript",
    [ENTITY_TYPE.LINK]: "Link",
    [ENTITY_TYPE.IMAGE]: "Image",
    [ENTITY_TYPE.HORIZONTAL_RULE]: "Horizontal line",
    [BR_TYPE]: "Line break",
    [UNDO_TYPE]: "Undo",
    [REDO_TYPE]: "Redo",
};
const KEYBOARD_SHORTCUTS = {
    [BLOCK_TYPE.UNSTYLED]: "⌫",
    [BLOCK_TYPE.HEADER_ONE]: "#",
    [BLOCK_TYPE.HEADER_TWO]: "##",
    [BLOCK_TYPE.HEADER_THREE]: "###",
    [BLOCK_TYPE.HEADER_FOUR]: "####",
    [BLOCK_TYPE.HEADER_FIVE]: "#####",
    [BLOCK_TYPE.HEADER_SIX]: "######",
    [BLOCK_TYPE.UNORDERED_LIST_ITEM]: "-",
    [BLOCK_TYPE.ORDERED_LIST_ITEM]: "1.",
    [BLOCK_TYPE.BLOCKQUOTE]: ">",
    [BLOCK_TYPE.CODE]: "```",
    [INLINE_STYLE.BOLD]: { other: "Ctrl + B", macOS: "⌘ + B" },
    [INLINE_STYLE.ITALIC]: { other: "Ctrl + I", macOS: "⌘ + I" },
    [INLINE_STYLE.UNDERLINE]: {
        other: "Ctrl + U",
        macOS: "⌘ + U",
    },
    [INLINE_STYLE.STRIKETHROUGH]: {
        other: "Ctrl + ⇧ + X",
        macOS: "⌘ + ⇧ + X",
    },
    [INLINE_STYLE.SUPERSCRIPT]: {
        other: "Ctrl + .",
        macOS: "⌘ + .",
    },
    [INLINE_STYLE.SUBSCRIPT]: {
        other: "Ctrl + ,",
        macOS: "⌘ + ,",
    },
    [ENTITY_TYPE.LINK]: { other: "Ctrl + K", macOS: "⌘ + K" },
    [BR_TYPE]: "⇧ + ↵",
    [ENTITY_TYPE.HORIZONTAL_RULE]: "- - -",
    [UNDO_TYPE]: { other: "Ctrl + Z", macOS: "⌘ + Z" },
    [REDO_TYPE]: { other: "Ctrl + ⇧ + Z", macOS: "⌘ + ⇧ + Z" },
};
const HANDLED = "handled";
const NOT_HANDLED = "not-handled";

/**
 * Inspired by draftjs-utils, with our custom functions.
 *
 * DraftUtils functions are utility helpers useful in isolation, specific to the Draft.js API,
 * without ties to Draftail's specific behavior or other APIs.
 */
var DraftUtils = {
    /**
     * Returns the first selected block.
     */
    getSelectedBlock(editorState) {
        const selection = editorState.getSelection();
        const content = editorState.getCurrentContent();
        return content.getBlockMap().get(selection.getStartKey());
    },
    /**
     * Returns the entity applicable to whole of current selection.
     * An entity can not span multiple blocks.
     * https://github.com/jpuri/draftjs-utils/blob/e81c0ae19c3b0fdef7e0c1b70d924398956be126/js/inline.js#L75
     */
    getSelectionEntity(editorState) {
        let entity;
        const selection = editorState.getSelection();
        let start = selection.getStartOffset();
        let end = selection.getEndOffset();
        if (start === end && start === 0) {
            end = 1;
        }
        else if (start === end) {
            start -= 1;
        }
        const block = this.getSelectedBlock(editorState);
        for (let i = start; i < end; i += 1) {
            const currentEntity = block.getEntityAt(i);
            if (!currentEntity) {
                entity = undefined;
                break;
            }
            if (i === start) {
                entity = currentEntity;
            }
            else if (entity !== currentEntity) {
                entity = undefined;
                break;
            }
        }
        return entity;
    },
    /**
     * Creates a selection on a given entity in the currently selected block.
     * Returns the current selection if no entity key is provided, or if the entity could not be found.
     */
    getEntitySelection(editorState, entityKey) {
        const selection = editorState.getSelection();
        if (!entityKey) {
            return selection;
        }
        const block = this.getSelectedBlock(editorState);
        let entityRange;
        // https://github.com/jpuri/draftjs-utils/blob/e81c0ae19c3b0fdef7e0c1b70d924398956be126/js/inline.js#L111
        block.findEntityRanges((value) => value.get("entity") === entityKey, (start, end) => {
            entityRange = {
                start,
                end,
            };
        });
        if (!entityRange) {
            return selection;
        }
        return selection.merge({
            anchorOffset: selection.isBackward ? entityRange.end : entityRange.start,
            focusOffset: selection.isBackward ? entityRange.start : entityRange.end,
        });
    },
    /**
     * Updates a given atomic block's entity, merging new data with the old one.
     */
    updateBlockEntity(editorState, block, data) {
        const content = editorState.getCurrentContent();
        let nextContent = content.mergeEntityData(block.getEntityAt(0), data);
        // To remove in Draft.js 0.11.
        // This is necessary because entity data is still using a mutable, global store.
        nextContent = Modifier.mergeBlockData(nextContent, new SelectionState({
            anchorKey: block.getKey(),
            anchorOffset: 0,
            focusKey: block.getKey(),
            focusOffset: block.getLength(),
        }), {});
        return EditorState.push(editorState, nextContent, "apply-entity");
    },
    /**
     * Inserts a horizontal rule in the place of the current selection.
     * Returns updated EditorState.
     * Inspired by DraftUtils.addLineBreakRemovingSelection.
     */
    addHorizontalRuleRemovingSelection(editorState) {
        const contentState = editorState.getCurrentContent();
        const contentStateWithEntity = contentState.createEntity(ENTITY_TYPE.HORIZONTAL_RULE, "IMMUTABLE", {});
        const entityKey = contentStateWithEntity.getLastCreatedEntityKey();
        return AtomicBlockUtils.insertAtomicBlock(editorState, entityKey, " ");
    },
    /**
     * Changes a block type to be `newType`, setting its new text.
     * Also removes the required characters from the characterList,
     * and resets block data.
     */
    resetBlockWithType(editorState, newType, newText, newData = {}) {
        const contentState = editorState.getCurrentContent();
        const selectionState = editorState.getSelection();
        const key = selectionState.getStartKey();
        const blockMap = contentState.getBlockMap();
        const block = blockMap.get(key);
        // Maintain persistence in the list while removing chars from the start.
        // https://github.com/facebook/draft-js/blob/788595984da7c1e00d1071ea82b063ff87140be4/src/model/transaction/removeRangeFromContentState.js#L333
        let chars = block.getCharacterList();
        let startOffset = 0;
        const sliceOffset = block.getText().length - newText.length;
        while (startOffset < sliceOffset) {
            chars = chars.shift();
            startOffset += 1;
        }
        const newBlock = block.merge({
            type: newType,
            text: newText,
            characterList: chars,
            data: newData,
        });
        const newContentState = contentState.merge({
            blockMap: blockMap.set(key, newBlock),
        });
        const newSelectionState = selectionState.merge({
            anchorOffset: 0,
            focusOffset: 0,
        });
        return EditorState.acceptSelection(EditorState.set(editorState, {
            currentContent: newContentState,
        }), newSelectionState);
    },
    /**
     * Applies an inline style on a given range, based on a Markdown shortcut,
     * removing the Markdown markers.
     * Supports adding styles on existing styles, and entities.
     */
    applyMarkdownStyle(editorState, range, char) {
        const selection = editorState.getSelection();
        let content = editorState.getCurrentContent();
        const marked = selection.merge({
            anchorOffset: range.start,
            focusOffset: range.end,
        });
        const endMarker = selection.merge({
            anchorOffset: range.end - range.pattern.length,
            focusOffset: range.end,
        });
        const startMarker = selection.merge({
            anchorOffset: range.start,
            focusOffset: range.start + range.pattern.length,
        });
        // Remove the markers separately to preserve existing styles and entities on the marked text.
        content = Modifier.applyInlineStyle(content, marked, range.type);
        content = Modifier.removeRange(content, endMarker, "forward");
        content = Modifier.removeRange(content, startMarker, "forward");
        const offset = selection.getFocusOffset() - range.pattern.length * 2;
        const endSelection = selection.merge({
            anchorOffset: offset,
            focusOffset: offset,
        });
        content = content.merge({ selectionAfter: endSelection });
        content = Modifier.insertText(content, endSelection, char);
        return EditorState.push(editorState, content, "change-inline-style");
    },
    /**
     * Removes the block at the given key.
     */
    removeBlock(editorState, key) {
        const content = editorState.getCurrentContent();
        const blockMap = content.getBlockMap().remove(key);
        return EditorState.set(editorState, {
            currentContent: content.merge({
                blockMap,
            }),
        });
    },
    /**
     * Removes a block-level entity, turning the block into an empty paragraph,
     * and placing the selection on it.
     */
    removeBlockEntity(editorState, entityKey, blockKey) {
        let newState = editorState;
        const content = editorState.getCurrentContent();
        const blockMap = content.getBlockMap();
        const block = blockMap.get(blockKey);
        const newBlock = block.merge({
            type: BLOCK_TYPE.UNSTYLED,
            text: "",
            // No text = no character list
            characterList: block.getCharacterList().slice(0, 0),
            data: {},
        });
        const newSelection = new SelectionState({
            anchorKey: blockKey,
            focusKey: blockKey,
            anchorOffset: 0,
            focusOffset: 0,
        });
        const newContent = content.merge({
            blockMap: blockMap.set(blockKey, newBlock),
        });
        newState = EditorState.push(newState, newContent, "change-block-type");
        newState = EditorState.forceSelection(newState, newSelection);
        return newState;
    },
    /**
     * Handles pressing delete within an atomic block. This can happen when selection is placed on an image.
     * Ideally this should be handled by the built-in RichUtils, but it's not.
     * See https://github.com/wagtail/wagtail/issues/4370.
     */
    handleDeleteAtomic(editorState) {
        const selection = editorState.getSelection();
        const content = editorState.getCurrentContent();
        const key = selection.getAnchorKey();
        const offset = selection.getAnchorOffset();
        const block = content.getBlockForKey(key);
        // Problematic selection. Pressing delete here would remove the entity, but not the block.
        if (selection.isCollapsed() &&
            block.getType() === BLOCK_TYPE.ATOMIC &&
            offset === 0) {
            return this.removeBlockEntity(editorState, block.getEntityAt(0), key);
        }
        return false;
    },
    /**
     * Get an entity decorator strategy based on the given entity type.
     * This strategy will find all entities of the given type.
     */
    getEntityTypeStrategy(entityType) {
        const strategy = (block, callback, contentState) => {
            block.findEntityRanges((character) => {
                const entityKey = character.getEntity();
                return (entityKey !== null &&
                    contentState.getEntity(entityKey).getType() === entityType);
            }, callback);
        };
        return strategy;
    },
    /**
     * Determines whether the editor should show its placeholder.
     * See https://draftjs.org/docs/api-reference-editor.html#placeholder
     * for details on why this is useful.
     */
    shouldHidePlaceholder(editorState) {
        const contentState = editorState.getCurrentContent();
        return (contentState.hasText() ||
            contentState.getBlockMap().first().getType() !== BLOCK_TYPE.UNSTYLED);
    },
    /**
     * Inserts new unstyled block.
     * Initially inspired from https://github.com/jpuri/draftjs-utils/blob/e81c0ae19c3b0fdef7e0c1b70d924398956be126/js/block.js#L153,
     * but changed so that the split + block type reset amounts to
     * only one change in the undo stack.
     */
    insertNewUnstyledBlock(editorState) {
        const selection = editorState.getSelection();
        let newContent = Modifier.splitBlock(editorState.getCurrentContent(), selection);
        const blockMap = newContent.getBlockMap();
        const blockKey = selection.getStartKey();
        const insertedBlockKey = newContent.getKeyAfter(blockKey);
        const newBlock = blockMap
            .get(insertedBlockKey)
            .set("type", BLOCK_TYPE.UNSTYLED);
        newContent = newContent.merge({
            blockMap: blockMap.set(insertedBlockKey, newBlock),
        });
        return EditorState.push(editorState, newContent, "split-block");
    },
    /**
     * Handles Shift + Enter keypress removing selection and inserting a line break.
     * https://github.com/jpuri/draftjs-utils/blob/112bbe449cc9156522fcf2b40f2910a071b795c2/js/block.js#L133
     */
    addLineBreak(editorState) {
        const content = editorState.getCurrentContent();
        const selection = editorState.getSelection();
        if (selection.isCollapsed()) {
            return RichUtils.insertSoftNewline(editorState);
        }
        let newContent = Modifier.removeRange(content, selection, "forward");
        const fragment = newContent.getSelectionAfter();
        const block = newContent.getBlockForKey(fragment.getStartKey());
        newContent = Modifier.insertText(newContent, fragment, "\n", block.getInlineStyleAt(fragment.getStartOffset()), null);
        return EditorState.push(editorState, newContent, "insert-fragment");
    },
    /**
     * Handles hard newlines.
     * https://github.com/jpuri/draftjs-utils/blob/e81c0ae19c3b0fdef7e0c1b70d924398956be126/js/keyPress.js#L17
     */
    handleHardNewline(editorState) {
        const selection = editorState.getSelection();
        if (!selection.isCollapsed()) {
            return false;
        }
        const content = editorState.getCurrentContent();
        const blockKey = selection.getStartKey();
        const block = content.getBlockForKey(blockKey);
        const blockType = block.getType();
        // Use a loose check to allow custom list item types to reuse the continuation behavior.
        const isListBlock = blockType.endsWith("-list-item");
        if (!isListBlock &&
            block.getType() !== BLOCK_TYPE.UNSTYLED &&
            block.getLength() === selection.getStartOffset()) {
            return this.insertNewUnstyledBlock(editorState);
        }
        if (isListBlock && block.getLength() === 0) {
            const depth = block.getDepth();
            if (depth === 0) {
                const nextContent = RichUtils.tryToRemoveBlockStyle(editorState);
                // At the moment, tryToRemoveBlockStyle always returns for
                // collapsed selections at the start of a block. So in theory this corner case should never happen.
                return nextContent
                    ? EditorState.push(editorState, nextContent, "change-block-type")
                    : false;
            }
            const blockMap = content.getBlockMap();
            const newBlock = block.set("depth", depth - 1);
            return EditorState.push(editorState, content.merge({
                blockMap: blockMap.set(blockKey, newBlock),
            }), "adjust-depth");
        }
        return false;
    },
    /**
     * Handles three scenarios:
     * - Soft newlines.
     * - Hard newlines in the "defer breaking out of the block" case.
     * - Other hard newlines.
     * See https://github.com/springload/draftail/issues/104,
     * https://github.com/jpuri/draftjs-utils/issues/10.
     */
    handleNewLine(editorState, event) {
        // https://github.com/jpuri/draftjs-utils/blob/e81c0ae19c3b0fdef7e0c1b70d924398956be126/js/keyPress.js#L64
        if (isSoftNewlineEvent(event)) {
            return this.addLineBreak(editorState);
        }
        const content = editorState.getCurrentContent();
        const selection = editorState.getSelection();
        const key = selection.getStartKey();
        const offset = selection.getStartOffset();
        const block = content.getBlockForKey(key);
        const isDeferredBreakoutBlock = [BLOCK_TYPE.CODE].includes(block.getType());
        if (isDeferredBreakoutBlock) {
            const isEmpty = selection.isCollapsed() && offset === 0 && block.getLength() === 0;
            if (isEmpty) {
                return EditorState.push(editorState, Modifier.setBlockType(content, selection, BLOCK_TYPE.UNSTYLED), "change-block-type");
            }
            return false;
        }
        return this.handleHardNewline(editorState);
    },
};

const { hasCommandModifier, isOptionKeyCommand } = KeyBindingUtil;
const hasCmd = hasCommandModifier;
// Hack relying on the internals of Draft.js.
// See https://github.com/facebook/draft-js/pull/869
// @ts-expect-error
const IS_MAC_OS = isOptionKeyCommand({ altKey: "test" }) === "test";
/**
 * Methods defining the behavior of the editor, depending on its configuration.
 */
var behavior = {
    /**
     * Configure block render map from block types list.
     */
    getBlockRenderMap(blockTypes) {
        let renderMap = DefaultDraftBlockRenderMap;
        // Override default element for code block.
        // Fix https://github.com/facebook/draft-js/issues/406.
        if (blockTypes.some((block) => block.type === BLOCK_TYPE.CODE)) {
            renderMap = renderMap.set(BLOCK_TYPE.CODE, {
                element: "code",
                wrapper: DefaultDraftBlockRenderMap.get(BLOCK_TYPE.CODE).wrapper,
            });
        }
        blockTypes
            .filter((block) => block.element)
            .forEach((block) => {
            renderMap = renderMap.set(block.type, {
                element: block.element,
            });
        });
        return renderMap;
    },
    /**
     * block style function automatically adding a class with the block's type.
     */
    blockStyleFn(block) {
        const type = block.getType();
        return `Draftail-block--${type} ${blockDepthStyleFn(block)}`;
    },
    /**
     * Configure key binding function from enabled blocks, styles, entities.
     */
    getKeyBindingFn(blockTypes, inlineStyles, entityTypes) {
        const getEnabled = (activeTypes) => activeTypes.reduce((enabled, type) => {
            enabled[type.type] = type.type;
            return enabled;
        }, {});
        const blocks = getEnabled(blockTypes);
        const styles = getEnabled(inlineStyles);
        const entities = getEnabled(entityTypes);
        // Emits key commands to use in `handleKeyCommand` in `Editor`.
        const keyBindingFn = (e) => {
            // Safeguard that we only trigger shortcuts with exact matches.
            // eg. cmd + shift + b should not trigger bold.
            if (e.shiftKey) {
                // Key bindings supported by Draft.js must be explicitely discarded.
                // See https://github.com/facebook/draft-js/issues/941.
                switch (e.keyCode) {
                    case KEY_CODES.B:
                        return undefined;
                    case KEY_CODES.I:
                        return undefined;
                    case KEY_CODES.J:
                        return undefined;
                    case KEY_CODES.U:
                        return undefined;
                    case KEY_CODES.X:
                        return hasCmd(e) && styles[INLINE_STYLE.STRIKETHROUGH];
                    case KEY_CODES[7]:
                        return hasCmd(e) && blocks[BLOCK_TYPE.ORDERED_LIST_ITEM];
                    case KEY_CODES[8]:
                        return hasCmd(e) && blocks[BLOCK_TYPE.UNORDERED_LIST_ITEM];
                    default:
                        return getDefaultKeyBinding(e);
                }
            }
            const ctrlAlt = (e.ctrlKey || e.metaKey) && e.altKey;
            switch (e.keyCode) {
                case KEY_CODES.K:
                    return hasCmd(e) && entities.LINK;
                case KEY_CODES.B:
                    return hasCmd(e) && styles[INLINE_STYLE.BOLD];
                case KEY_CODES.I:
                    return hasCmd(e) && styles[INLINE_STYLE.ITALIC];
                case KEY_CODES.J:
                    return hasCmd(e) && styles[INLINE_STYLE.CODE];
                case KEY_CODES.U:
                    return hasCmd(e) && styles[INLINE_STYLE.UNDERLINE];
                case KEY_CODES["."]:
                    return hasCmd(e) && styles[INLINE_STYLE.SUPERSCRIPT];
                case KEY_CODES[","]:
                    return hasCmd(e) && styles[INLINE_STYLE.SUBSCRIPT];
                case KEY_CODES[0]:
                    // Reverting to unstyled block is always available.
                    return ctrlAlt && BLOCK_TYPE.UNSTYLED;
                case KEY_CODES[1]:
                    return ctrlAlt && blocks[BLOCK_TYPE.HEADER_ONE];
                case KEY_CODES[2]:
                    return ctrlAlt && blocks[BLOCK_TYPE.HEADER_TWO];
                case KEY_CODES[3]:
                    return ctrlAlt && blocks[BLOCK_TYPE.HEADER_THREE];
                case KEY_CODES[4]:
                    return ctrlAlt && blocks[BLOCK_TYPE.HEADER_FOUR];
                case KEY_CODES[5]:
                    return ctrlAlt && blocks[BLOCK_TYPE.HEADER_FIVE];
                case KEY_CODES[6]:
                    return ctrlAlt && blocks[BLOCK_TYPE.HEADER_SIX];
                default:
                    return getDefaultKeyBinding(e);
            }
        };
        return keyBindingFn;
    },
    hasKeyboardShortcut(type) {
        return !!KEYBOARD_SHORTCUTS[type];
    },
    getKeyboardShortcut(type, isMacOS = IS_MAC_OS) {
        const shortcut = KEYBOARD_SHORTCUTS[type];
        const system = isMacOS ? "macOS" : "other";
        return (shortcut && shortcut[system]) || shortcut;
    },
    /**
     * Defines whether a block should be altered to a new type when
     * the user types a given mark.
     * This powers the "autolist" feature.
     *
     * Returns the new block type, or false if no replacement should occur.
     */
    handleBeforeInputBlockType(mark, blockTypes) {
        return blockTypes.find((b) => b.type === INPUT_BLOCK_MAP[mark])
            ? INPUT_BLOCK_MAP[mark]
            : false;
    },
    handleBeforeInputHR(mark, block) {
        return (mark === INPUT_ENTITY_MAP[ENTITY_TYPE.HORIZONTAL_RULE] &&
            block.getType() !== BLOCK_TYPE.CODE);
    },
    /**
     * Checks whether a given input string contains style shortcuts.
     * If so, returns the range onto which the shortcut is applied.
     */
    handleBeforeInputInlineStyle(input, inlineStyles) {
        const activeShortcuts = INPUT_STYLE_MAP.filter(({ type }) => inlineStyles.some((s) => s.type === type));
        let range;
        const match = activeShortcuts.find(({ regex }) => {
            // Re-create a RegExp object every time because RegExp is stateful.
            range = new RegExp(regex, "g").exec(input);
            return range;
        });
        return range && match
            ? {
                pattern: match.pattern,
                start: range.index === 0 ? 0 : range.index + 1,
                end: range.index + range[0].length,
                type: match.type,
            }
            : false;
    },
    getCustomStyleMap(inlineStyles) {
        const customStyleMap = {};
        inlineStyles.forEach((style) => {
            if (style.style) {
                customStyleMap[style.type] = style.style;
            }
            else if (CUSTOM_STYLE_MAP[style.type]) {
                customStyleMap[style.type] = CUSTOM_STYLE_MAP[style.type];
            }
            else {
                customStyleMap[style.type] = {};
            }
        });
        return customStyleMap;
    },
    /**
     * Applies whitelist and blacklist operations to the editor content,
     * so the resulting editor state is shaped according to Draftail
     * expectations and configuration.
     */
    filterPaste({ maxListNesting, enableHorizontalRule, enableLineBreak, blockTypes, inlineStyles, entityTypes, }, editorState) {
        const enabledEntityTypes = entityTypes.slice();
        const whitespacedCharacters = ["\t", "📷"];
        if (enableHorizontalRule) {
            enabledEntityTypes.push({
                type: ENTITY_TYPE.HORIZONTAL_RULE,
            });
        }
        if (!enableLineBreak) {
            whitespacedCharacters.push("\n");
        }
        return filterEditorState({
            blocks: blockTypes.map((b) => b.type),
            styles: inlineStyles.map((s) => s.type),
            entities: enabledEntityTypes,
            maxNesting: maxListNesting,
            whitespacedCharacters,
        }, editorState);
    },
};

/**
 * Icon as SVG element. Can optionally render a React element instead.
 */
const Icon = ({ icon, title, className }) => {
    let children;
    if (typeof icon === "string") {
        if (icon.includes("#")) {
            children = React.createElement("use", { xlinkHref: icon });
        }
        else {
            children = React.createElement("path", { d: icon });
        }
    }
    else if (Array.isArray(icon)) {
        // eslint-disable-next-line @thibaudcolas/cookbook/react/no-array-index-key
        children = icon.map((d, i) => React.createElement("path", { key: i, d: d }));
    }
    else {
        return icon;
    }
    return (React.createElement("svg", { width: "16", height: "16", viewBox: "0 0 1024 1024", className: `Draftail-Icon ${className || ""}`, "aria-hidden": title ? undefined : "true", role: title ? "img" : undefined, "aria-label": title || undefined }, children));
};

/**
 * Displays a basic button, with optional active variant,
 * enriched with a tooltip. The tooltip stops showing on click.
 */
class ToolbarButton extends PureComponent {
    constructor(props) {
        super(props);
        this.state = {
            showTooltipOnHover: true,
        };
        this.onMouseDown = this.onMouseDown.bind(this);
        this.onMouseLeave = this.onMouseLeave.bind(this);
    }
    onMouseDown(e) {
        const { name, onClick } = this.props;
        e.preventDefault();
        this.setState({
            showTooltipOnHover: false,
        });
        if (onClick) {
            onClick(name || "");
        }
    }
    onMouseLeave() {
        this.setState({
            showTooltipOnHover: true,
        });
    }
    render() {
        const { name, active, label, title, icon } = this.props;
        const { showTooltipOnHover } = this.state;
        return (React.createElement("button", { name: name, className: `Draftail-ToolbarButton${active ? " Draftail-ToolbarButton--active" : ""}`, type: "button", "aria-label": title, "data-draftail-balloon": title && showTooltipOnHover ? true : null, tabIndex: -1, onMouseDown: this.onMouseDown, onMouseLeave: this.onMouseLeave },
            icon ? React.createElement(Icon, { icon: icon }) : null,
            label ? (React.createElement("span", { className: "Draftail-ToolbarButton__label" }, label)) : null));
    }
}

const ToolbarGroup = ({ children }) => {
    const hasChildren = React.Children.toArray(children).some((c) => c !== null);
    return hasChildren ? (React.createElement("div", { className: "Draftail-ToolbarGroup" }, children)) : null;
};
ToolbarGroup.defaultProps = {
    children: null,
};

const getControlLabel = (type, config) => {
    const predefinedType = type;
    if (typeof config === "boolean") {
        return LABELS[predefinedType];
    }
    if (typeof config.label === "string" || config.label === null) {
        return config.label;
    }
    if (typeof config.icon !== "undefined") {
        return null;
    }
    return LABELS[predefinedType];
};
const getControlDescription = (control) => {
    const predefinedType = control.type;
    const useDefaultDescription = typeof control.description === "undefined";
    const defaultDescription = DESCRIPTIONS[predefinedType];
    const description = useDefaultDescription
        ? defaultDescription
        : control.description;
    const useDefaultLabel = typeof control.label === "undefined";
    const defaultLabel = LABELS[predefinedType];
    const label = useDefaultLabel ? defaultLabel : control.label;
    return description || label;
};

const showButton$1 = (config) => Boolean(config.icon) || Boolean(getControlLabel(config.type, config));
const getButtonTitle$1 = (type, config) => {
    const description = typeof config === "boolean" || typeof config.description === "undefined"
        ? DESCRIPTIONS[type]
        : config.description;
    const hasShortcut = behavior.hasKeyboardShortcut(type);
    let title = description;
    if (hasShortcut) {
        const desc = description ? `${description}\n` : "";
        title = `${desc}${behavior.getKeyboardShortcut(type)}`;
    }
    return title;
};
class ToolbarDefaults$1 extends PureComponent {
    render() {
        const { currentStyles, currentBlock, blockTypes, inlineStyles, enableHorizontalRule, enableLineBreak, showUndoControl, showRedoControl, entityTypes, toggleBlockType, toggleInlineStyle, addHR, addBR, onUndoRedo, onRequestSource, } = this.props;
        return [
            React.createElement(ToolbarGroup, { key: "styles" }, inlineStyles.filter(showButton$1).map((t) => (React.createElement(ToolbarButton, { key: t.type, name: t.type, active: currentStyles.has(t.type), label: getControlLabel(t.type, t), title: getButtonTitle$1(t.type, t), icon: t.icon, onClick: toggleInlineStyle })))),
            React.createElement(ToolbarGroup, { key: "blocks" }, blockTypes.filter(showButton$1).map((t) => (React.createElement(ToolbarButton, { key: t.type, name: t.type, active: currentBlock === t.type, label: getControlLabel(t.type, t), title: getButtonTitle$1(t.type, t), icon: t.icon, onClick: toggleBlockType })))),
            React.createElement(ToolbarGroup, { key: "hr-br" },
                enableHorizontalRule ? (React.createElement(ToolbarButton, { name: ENTITY_TYPE.HORIZONTAL_RULE, onClick: addHR, label: getControlLabel(ENTITY_TYPE.HORIZONTAL_RULE, enableHorizontalRule), title: getButtonTitle$1(ENTITY_TYPE.HORIZONTAL_RULE, enableHorizontalRule), icon: typeof enableHorizontalRule !== "boolean"
                        ? enableHorizontalRule.icon
                        : null })) : null,
                enableLineBreak ? (React.createElement(ToolbarButton, { name: BR_TYPE, onClick: addBR, label: getControlLabel(BR_TYPE, enableLineBreak), title: getButtonTitle$1(BR_TYPE, enableLineBreak), icon: typeof enableLineBreak !== "boolean" ? enableLineBreak.icon : null })) : null),
            React.createElement(ToolbarGroup, { key: "entities" }, entityTypes.filter(showButton$1).map((t) => (React.createElement(ToolbarButton, { key: t.type, name: t.type, onClick: onRequestSource, label: getControlLabel(t.type, t), title: getButtonTitle$1(t.type, t), icon: t.icon })))),
            React.createElement(ToolbarGroup, { key: "undo-redo" },
                showUndoControl ? (React.createElement(ToolbarButton, { name: UNDO_TYPE, onClick: onUndoRedo, label: getControlLabel(UNDO_TYPE, showUndoControl), title: getButtonTitle$1(UNDO_TYPE, showUndoControl), icon: typeof showUndoControl !== "boolean" ? showUndoControl.icon : null })) : null,
                showRedoControl ? (React.createElement(ToolbarButton, { name: REDO_TYPE, onClick: onUndoRedo, label: getControlLabel(REDO_TYPE, showRedoControl), title: getButtonTitle$1(REDO_TYPE, showRedoControl), icon: typeof showRedoControl !== "boolean" ? showRedoControl.icon : null })) : null),
        ];
    }
}

const Toolbar = (props) => {
    // Support the legacy and current controls APIs.
    const { controls, getEditorState, onChange } = props;
    return (React.createElement("div", { className: "Draftail-Toolbar", role: "toolbar" },
        React.createElement(ToolbarDefaults$1, { ...props }),
        React.createElement(ToolbarGroup, null, controls.map((control, i) => {
            if (control.inline || control.meta) {
                return null;
            }
            const Control = control.block || control;
            return (React.createElement(Control
            // eslint-disable-next-line @thibaudcolas/cookbook/react/no-array-index-key
            , { 
                // eslint-disable-next-line @thibaudcolas/cookbook/react/no-array-index-key
                key: i, getEditorState: getEditorState, onChange: onChange }));
        }))));
};

/**
 * Generates CSS styles for list items, for a given depth.
 * @param {number} max
 */
function Styles$1({ max }) {
    return React.createElement("style", null, getListNestingStyles(max));
}
const ListNestingStyles = React.memo(Styles$1);

/**
 * An <hr/> in the editor.
 */
const DividerBlock = () => React.createElement("hr", { className: "Draftail-DividerBlock" });

function ComboBox({ label, placeholder, items, onSelect, }) {
    const [inputItems, setInputItems] = useState(items);
    const { getLabelProps, getMenuProps, getInputProps, getComboboxProps, getItemProps, reset, selectItem, setHighlightedIndex, } = useCombobox({
        items: inputItems,
        itemToString(item) {
            if (!item) {
                return "";
            }
            return (getControlDescription(item) || getControlLabel(item.type, item) || "");
        },
        selectedItem: null,
        onSelectedItemChange: onSelect,
        onInputValueChange: (changes) => {
            const { inputValue } = changes;
            if (!inputValue) {
                setInputItems(items);
                return;
            }
            const input = inputValue.toLowerCase();
            const filtered = items.filter((item) => {
                const label = getControlLabel(item.type, item);
                if (label) {
                    const matchLabel = label.toLowerCase().startsWith(input);
                    if (matchLabel) {
                        return matchLabel;
                    }
                }
                const description = getControlDescription(item);
                if (description) {
                    return description.toLowerCase().startsWith(input);
                }
            });
            setInputItems(filtered);
            // Always reset the first item to highlighted on filtering, to speed up selection.
            setHighlightedIndex(0);
        },
    });
    return (React.createElement("div", { className: `Draftail-ComboBox Draftail-ComboBox--${label ? "field" : "contenteditable"}` },
        React.createElement("label", { className: "Draftail-ComboBox__label", ...getLabelProps() }, label),
        React.createElement("div", { ...getComboboxProps() },
            React.createElement("input", { ...getInputProps(), placeholder: placeholder })),
        React.createElement("div", { ...getMenuProps() }, inputItems.map((item, index) => {
            const label = getControlLabel(item.type, item);
            const description = getControlDescription(item);
            return (React.createElement("div", { key: `${label}${item.type}${index}`, ...getItemProps({ item, index }) },
                React.createElement("div", { className: "Draftail-ComboBox__option-icon" },
                    typeof item.icon !== "undefined" && item.icon !== null ? (React.createElement(Icon, { icon: item.icon })) : null,
                    label ? React.createElement("span", null, label) : null),
                React.createElement("div", { className: "Draftail-ComboBox__option-text" }, description)));
        }))));
}

const getReferenceClientRect$1 = () => getVisibleSelectionRect(window);
const CommandComboBox = ({ textDirectionality, currentBlockKey, currentBlock, blockTypes, match, getEditorState, onCompleteSource, }) => {
    const commands = blockTypes.map((t) => ({
        ...t,
        onSelect: () => {
            return RichUtils.toggleBlockType(getEditorState(), t.type);
        },
    }));
    const tippyParentRef = useRef(null);
    const [selectionRect, setSelectionRect] = useState();
    const editorState = getEditorState();
    const selection = editorState.getSelection();
    const isCollapsed = selection.isCollapsed();
    useEffect(() => {
        if (isCollapsed) {
            setSelectionRect(getReferenceClientRect$1());
        }
        else {
            setSelectionRect(null);
        }
    }, [isCollapsed]);
    const isVisible = isCollapsed && Boolean(selectionRect);
    return (React.createElement("div", { className: "Draftail-CommandPalette" },
        isVisible ? (React.createElement(Tippy, { visible: isVisible, getReferenceClientRect: () => selectionRect, maxWidth: "100%", interactive: true, arrow: false, placement: "bottom", appendTo: () => tippyParentRef.current, content: React.createElement(ComboBox, { key: `${currentBlockKey}-${currentBlock}`, items: commands, onSelect: (selection) => {
                    setSelectionRect(null);
                    onCompleteSource(selection.selectedItem.onSelect());
                } }) })) : null,
        React.createElement("div", { ref: tippyParentRef })));
};

const CommandPalette = ({ textDirectionality, blockTypes, getEditorState, currentBlock, currentBlockKey, onCompleteSource, }) => {
    const editorState = getEditorState();
    editorState.getSelection();
    const block = DraftUtils.getSelectedBlock(editorState);
    const text = block.getText();
    if (!text.startsWith("/")) {
        return null;
    }
    return (React.createElement(CommandComboBox, { blockTypes: blockTypes, currentBlock: currentBlock, currentBlockKey: currentBlockKey, match: text.replace("/", ""), getEditorState: getEditorState, onCompleteSource: onCompleteSource }));
};

/**
 * Adds placeholder "Write here…" text to the currently-focused block.
 * Done with CSS so this can switch blocks without re-rendering the whole editor.
 */
function Styles({ blockKey, placeholder }) {
    if (!blockKey || !placeholder) {
        return null;
    }
    return (React.createElement("style", null, `.Draftail-block--unstyled[data-offset-key="${blockKey}-0-0"] .Draftail-Placeholder::before { content: "${placeholder}"; }`));
}
const PlaceholderStyles = React.memo(Styles);

/**
 * Renders placeholder as a data-text attribute. This allows styling with
 * CSS `content: attr(data-text)`, which can bypass Draft.js rendering optimisations.
 */
function Placeholder({ text }) {
    return React.createElement("div", { className: "Draftail-Placeholder", "data-text": text });
}

const findBlockDescription = (type, blockTypes) => {
    const blockType = blockTypes.find((t) => t.type === type);
    return blockType ? getControlDescription(blockType) : null;
};
/**
 * A DraftEditorBlock, with additional placeholder text to display in some criteria.
 */
const DraftailPlaceholderBlock = (props) => {
    const { blockProps, ...otherProps } = props;
    const { blockTypes, ...otherBlockProps } = blockProps;
    const { block } = otherProps;
    const blockDescription = findBlockDescription(block.getType(), blockTypes);
    return (React.createElement(React.Fragment, null,
        React.createElement(Placeholder, { text: blockDescription }),
        React.createElement(DraftEditorBlock, { ...otherProps, blockProps: otherBlockProps })));
};

const defaultProps = {
    /** Initial content of the editor. Use this to edit pre-existing content. */
    rawContentState: null,
    /** Called when changes occurred. Use this to persist editor content. */
    onSave: null,
    /** Content of the editor, when using the editor as a controlled component. Incompatible with `rawContentState` and `onSave`. */
    editorState: null,
    /** Called whenever the editor state is updated. Use this to manage the content of a controlled editor. Incompatible with `rawContentState` and `onSave`. */
    onChange: null,
    /** Called when the editor receives focus. */
    onFocus: null,
    /** Called when the editor loses focus. */
    onBlur: null,
    /** Displayed when the editor is empty. Hidden if the user changes styling. */
    placeholder: null,
    /** Enable the use of horizontal rules in the editor. */
    enableHorizontalRule: false,
    /** Enable the use of line breaks in the editor. */
    enableLineBreak: false,
    /** Show undo control in the toolbar. */
    showUndoControl: false,
    /** Show redo control in the toolbar. */
    showRedoControl: false,
    /** Disable copy/paste of rich text in the editor. */
    stripPastedStyles: true,
    /** Set if the editor supports multiple lines / blocks of text, or only a single line. */
    multiline: true,
    /** Set whether spellcheck is turned on for your editor.
     * See https://draftjs.org/docs/api-reference-editor.html#spellcheck.
     */
    spellCheck: false,
    /** Set whether the editor should be rendered in readOnly mode.
     * See https://draftjs.org/docs/api-reference-editor.html#readonly
     */
    readOnly: false,
    /** Optionally set the overriding text alignment for this editor.
     * See https://draftjs.org/docs/api-reference-editor.html#textalignment.
     */
    textAlignment: null,
    /** Optionally set the overriding text directionality for this editor.
     * See https://draftjs.org/docs/api-reference-editor.html#textdirectionality.
     */
    textDirectionality: null,
    /** Set if auto capitalization is turned on and how it behaves.
     * See https://draftjs.org/docs/api-reference-editor.html#autocapitalize-string.
     */
    autoCapitalize: null,
    /** Set if auto complete is turned on and how it behaves.
     * See https://draftjs.org/docs/api-reference-editor.html#autocomplete-string.
     */
    autoComplete: null,
    /** Set if auto correct is turned on and how it behaves.
     * See https://draftjs.org/docs/api-reference-editor.html#autocorrect-string.
     */
    autoCorrect: null,
    /** See https://draftjs.org/docs/api-reference-editor.html#aria-props. */
    ariaDescribedBy: null,
    ariaExpanded: null,
    ariaLabel: null,
    ariaLabelledBy: null,
    ariaOwneeID: null,
    ariaRequired: null,
    /** List of the available block types. */
    blockTypes: [],
    /** List of the available inline styles. */
    inlineStyles: [],
    /** List of the available entity types. */
    entityTypes: [],
    /** List of active decorators. */
    decorators: [],
    /** List of extra toolbar controls. */
    controls: [],
    /** List of plugins of the draft-js-plugins architecture. */
    plugins: [],
    /** Optionally override the default Draftail toolbar, removing or replacing it. */
    topToolbar: Toolbar,
    /** Optionally add a custom toolbar underneath the editor, e.g. for metrics. */
    bottomToolbar: null,
    /** Max level of nesting for list items. 0 = no nesting. Maximum = 10. */
    maxListNesting: 1,
    /** Frequency at which to call the onSave callback (ms). */
    stateSaveInterval: 250,
};
/**
 * Main component of the Draftail editor.
 * Contains the Draft.js editor instance, and ties together UI and behavior.
 */
class DraftailEditor extends Component {
    constructor(props) {
        super(props);
        this.onChange = this.onChange.bind(this);
        this.saveState = this.saveState.bind(this);
        this.toggleSource = this.toggleSource.bind(this);
        this.toggleEditor = this.toggleEditor.bind(this);
        this.lockEditor = this.toggleEditor.bind(this, true);
        this.unlockEditor = this.toggleEditor.bind(this, false);
        this.handleReturn = this.handleReturn.bind(this);
        this.onFocus = this.onFocus.bind(this);
        this.onBlur = this.onBlur.bind(this);
        this.onTab = this.onTab.bind(this);
        this.onUpArrow = this.onUpArrow.bind(this);
        this.onDownArrow = this.onDownArrow.bind(this);
        this.handleKeyCommand = this.handleKeyCommand.bind(this);
        this.handleBeforeInput = this.handleBeforeInput.bind(this);
        this.handlePastedText = this.handlePastedText.bind(this);
        this.toggleBlockType = this.toggleBlockType.bind(this);
        this.toggleInlineStyle = this.toggleInlineStyle.bind(this);
        this.onEditEntity = this.onEditEntity.bind(this);
        this.onRemoveEntity = this.onRemoveEntity.bind(this);
        this.addHR = this.addHR.bind(this);
        this.addBR = this.addBR.bind(this);
        this.onUndoRedo = this.onUndoRedo.bind(this);
        this.blockRenderer = this.blockRenderer.bind(this);
        this.onRequestSource = this.onRequestSource.bind(this);
        this.onCompleteSource = this.onCompleteSource.bind(this);
        this.onCloseSource = this.onCloseSource.bind(this);
        this.focus = this.focus.bind(this);
        this.renderSource = this.renderSource.bind(this);
        this.renderCommandPalette = this.renderCommandPalette.bind(this);
        const { editorState, rawContentState } = props;
        this.state = {
            readOnlyState: false,
            hasFocus: false,
            sourceOptions: null,
        };
        if (editorState !== null) {
            this.getEditorState = this.getEditorStateProp.bind(this);
        }
        else {
            // If editorState is not used as a prop, create it in local state from rawContentState.
            this.state.editorState = createEditorStateFromRaw(rawContentState);
            this.getEditorState = this.getEditorStateState.bind(this);
        }
    }
    componentDidMount() {
        this.copySource = registerCopySource(this.editorRef.editor);
    }
    componentWillUnmount() {
        this.copySource.unregister();
        window.clearTimeout(this.updateTimeout);
    }
    onFocus() {
        this.setState({
            hasFocus: true,
        });
        const { onFocus } = this.props;
        if (onFocus) {
            onFocus();
        }
    }
    onBlur() {
        this.setState({
            hasFocus: false,
        });
        const { onBlur } = this.props;
        if (onBlur) {
            onBlur();
        }
    }
    onTab(event) {
        const { maxListNesting, blockTypes } = this.props;
        const editorState = this.getEditorState();
        const newState = RichUtils.onTab(event, editorState, maxListNesting);
        editorState.getSelection();
        const block = DraftUtils.getSelectedBlock(editorState);
        const text = block.getText();
        if (text === "/blockquote") {
            event.preventDefault();
            this.toggleBlockType("blockquote");
            return true;
        }
        if (text === "/hr") {
            const nextState = DraftUtils.removeBlock(DraftUtils.addHorizontalRuleRemovingSelection(editorState), block.getKey());
            event.preventDefault();
            this.onChange(nextState);
            return true;
        }
        this.onChange(newState);
        return true;
    }
    onUpArrow(event) {
        const input = document.querySelector('[aria-autocomplete="list"]');
        if (input) {
            const evt = new Event("keydown", { bubbles: true });
            evt.keyCode = 38;
            evt.key = "ArrowUp";
            input?.dispatchEvent(evt);
            event.preventDefault();
        }
    }
    onDownArrow(event) {
        const input = document.querySelector('[aria-autocomplete="list"]');
        if (input) {
            const evt = new Event("keydown", { bubbles: true });
            evt.keyCode = 40;
            evt.key = "ArrowDown";
            input?.dispatchEvent(evt);
            event.preventDefault();
        }
    }
    onChange(nextState) {
        const { multiline, stateSaveInterval, maxListNesting, enableHorizontalRule, enableLineBreak, blockTypes, inlineStyles, entityTypes, onChange, } = this.props;
        const prevState = this.getEditorState();
        const shouldFilterPaste = nextState.getCurrentContent() !== prevState.getCurrentContent() &&
            nextState.getLastChangeType() === "insert-fragment";
        let filteredState = nextState;
        if (shouldFilterPaste) {
            filteredState = behavior.filterPaste({
                maxListNesting,
                enableHorizontalRule,
                enableLineBreak,
                blockTypes,
                inlineStyles,
                entityTypes,
            }, filteredState);
            if (!multiline) {
                filteredState = condenseBlocks(filteredState, prevState);
            }
        }
        if (onChange) {
            onChange(filteredState);
        }
        else {
            this.setState({
                editorState: filteredState,
            }, () => {
                window.clearTimeout(this.updateTimeout);
                this.updateTimeout = window.setTimeout(this.saveState, stateSaveInterval);
            });
        }
    }
    onEditEntity(entityKey) {
        const { entityTypes } = this.props;
        const editorState = this.getEditorState();
        const content = editorState.getCurrentContent();
        const entity = content.getEntity(entityKey);
        const entityType = entityTypes.find((t) => t.type === entity.getType());
        if (!entityType.block) {
            const entitySelection = DraftUtils.getEntitySelection(editorState, entityKey);
            const nextState = EditorState.acceptSelection(editorState, entitySelection);
            this.onChange(nextState);
        }
        this.toggleSource(entity.getType(), entityKey, entity);
    }
    onRemoveEntity(entityKey, blockKey) {
        const { entityTypes } = this.props;
        const editorState = this.getEditorState();
        const content = editorState.getCurrentContent();
        const entity = content.getEntity(entityKey);
        const entityType = entityTypes.find((t) => t.type === entity.getType());
        let newState = editorState;
        if (entityType.block) {
            newState = DraftUtils.removeBlockEntity(newState, entityKey, blockKey);
        }
        else {
            const entitySelection = DraftUtils.getEntitySelection(editorState, entityKey);
            newState = RichUtils.toggleLink(newState, entitySelection, null);
        }
        this.onChange(newState);
    }
    onUndoRedo(type) {
        const editorState = this.getEditorState();
        let newEditorState = editorState;
        if (type === UNDO_TYPE) {
            newEditorState = EditorState.undo(editorState);
        }
        else if (type === REDO_TYPE) {
            newEditorState = EditorState.redo(editorState);
        }
        this.onChange(newEditorState);
    }
    onRequestSource(entityType) {
        const editorState = this.getEditorState();
        const contentState = editorState.getCurrentContent();
        const entityKey = DraftUtils.getSelectionEntity(editorState);
        this.toggleSource(entityType, entityKey, entityKey ? contentState.getEntity(entityKey) : null);
    }
    onCompleteSource(nextState) {
        this.setState({
            sourceOptions: null,
        }, () => {
            if (nextState) {
                this.onChange(nextState);
            }
            window.setTimeout(() => {
                this.setState({ readOnlyState: false }, () => {
                    window.setTimeout(() => {
                        this.focus();
                    }, 0);
                });
            }, 0);
        });
    }
    onCloseSource() {
        this.setState({
            sourceOptions: null,
            readOnlyState: false,
        });
    }
    getEditorStateProp() {
        const { editorState } = this.props;
        return editorState;
    }
    getEditorStateState() {
        const { editorState } = this.state;
        return editorState;
    }
    saveState() {
        const { onSave } = this.props;
        const editorState = this.getEditorState();
        if (onSave) {
            onSave(serialiseEditorStateToRaw(editorState));
        }
    }
    toggleEditor(readOnlyState) {
        this.setState({
            readOnlyState,
        });
    }
    toggleSource(type, entityKey, entity) {
        const { entityTypes } = this.props;
        const entityType = entityTypes.find((item) => item.type === type);
        this.setState({
            readOnlyState: true,
            sourceOptions: {
                entity,
                entityKey,
                entityType,
            },
        });
    }
    handleReturn(e) {
        const { multiline, enableLineBreak, inlineStyles } = this.props;
        const editorState = this.getEditorState();
        const input = document.querySelector('[aria-autocomplete="list"]');
        if (input) {
            const evt = new Event("keydown", { bubbles: true });
            evt.keyCode = 13;
            evt.key = "Enter";
            input?.dispatchEvent(evt);
            e.preventDefault();
            return HANDLED;
        }
        // alt + enter opens links and other entities with a `url` property.
        if (e.altKey) {
            const entityKey = DraftUtils.getSelectionEntity(editorState);
            if (entityKey) {
                const content = editorState.getCurrentContent();
                const entityData = content.getEntity(entityKey).getData();
                if (entityData.url) {
                    window.open(entityData.url);
                }
            }
            // Mark the return as handled even if there is no entity.
            // alt + enter should never create a newline anyway.
            return HANDLED;
        }
        if (!enableLineBreak) {
            // Quick hack to disable soft line breaks.
            e.which = 0;
        }
        let newState = editorState;
        let newStyle = false;
        const selection = newState.getSelection();
        // Check whether we should apply a Markdown styles shortcut.
        if (selection.isCollapsed()) {
            const block = DraftUtils.getSelectedBlock(editorState);
            newStyle = behavior.handleBeforeInputInlineStyle(block.getText(), inlineStyles);
            if (newStyle) {
                newState = DraftUtils.applyMarkdownStyle(newState, newStyle, "");
            }
        }
        if (!multiline) {
            return HANDLED;
        }
        const newLineState = DraftUtils.handleNewLine(newState, e);
        // Manually handle the return if there is a style to apply.
        if (!newLineState && newStyle) {
            const content = newState.getCurrentContent();
            const newContent = Modifier.splitBlock(content, selection);
            newState = EditorState.push(newState, newContent, "split-block");
            // Do not propagate the style from the last block.
            newState = RichUtils.toggleInlineStyle(newState, newStyle.type);
        }
        else {
            newState = newLineState;
        }
        if (newState && newState !== editorState) {
            this.onChange(newState);
            return HANDLED;
        }
        return NOT_HANDLED;
    }
    handleKeyCommand(command) {
        const { entityTypes, blockTypes, inlineStyles } = this.props;
        const editorState = this.getEditorState();
        if (entityTypes.some((t) => t.type === command)) {
            this.onRequestSource(command);
            return HANDLED;
        }
        if (blockTypes.some((t) => t.type === command)) {
            this.toggleBlockType(command);
            return HANDLED;
        }
        if (inlineStyles.some((t) => t.type === command)) {
            this.toggleInlineStyle(command);
            return HANDLED;
        }
        // Special case – some delete commands on atomic blocks are not covered by RichUtils.
        if (command === "delete") {
            const newState = DraftUtils.handleDeleteAtomic(editorState);
            if (newState) {
                this.onChange(newState);
                return HANDLED;
            }
        }
        // If the command is known but not enabled for this editor, treat it as handled but don't do anything.
        if (KEY_COMMANDS.includes(command)) {
            return HANDLED;
        }
        const newState = RichUtils.handleKeyCommand(editorState, command);
        if (newState) {
            this.onChange(newState);
            return HANDLED;
        }
        return NOT_HANDLED;
    }
    handleBeforeInput(char) {
        const { blockTypes, inlineStyles, enableHorizontalRule } = this.props;
        const editorState = this.getEditorState();
        const selection = editorState.getSelection();
        if (selection.isCollapsed()) {
            const block = DraftUtils.getSelectedBlock(editorState);
            const startOffset = selection.getStartOffset();
            const text = block.getText();
            const beforeInput = text.slice(0, startOffset);
            const mark = `${beforeInput}${char}`;
            let newEditorState = editorState;
            const newBlockType = behavior.handleBeforeInputBlockType(mark, blockTypes);
            if (newBlockType) {
                newEditorState = DraftUtils.resetBlockWithType(newEditorState, newBlockType, text.replace(beforeInput, ""));
            }
            if (enableHorizontalRule && behavior.handleBeforeInputHR(mark, block)) {
                newEditorState = DraftUtils.removeBlock(DraftUtils.addHorizontalRuleRemovingSelection(newEditorState), block.getKey());
            }
            const newStyle = behavior.handleBeforeInputInlineStyle(beforeInput, inlineStyles);
            if (newStyle) {
                newEditorState = DraftUtils.applyMarkdownStyle(newEditorState, newStyle, char);
            }
            if (newEditorState !== editorState) {
                this.onChange(newEditorState);
                return HANDLED;
            }
        }
        return NOT_HANDLED;
    }
    handlePastedText(text, html, editorState) {
        const { stripPastedStyles, entityTypes } = this.props;
        const hasProcessedEntity = entityTypes.some((t) => {
            const ret = t.onPaste &&
                t.onPaste(text, html, editorState, {
                    setEditorState: this.onChange,
                    getEditorState: this.getEditorState,
                }, t);
            return ret === HANDLED;
        });
        if (hasProcessedEntity) {
            return HANDLED;
        }
        // Leave paste handling to Draft.js when stripping styles is desirable.
        if (stripPastedStyles) {
            return NOT_HANDLED;
        }
        const pastedState = handleDraftEditorPastedText(html, editorState);
        if (pastedState) {
            this.onChange(pastedState);
            return HANDLED;
        }
        return NOT_HANDLED;
    }
    toggleBlockType(blockType) {
        const editorState = this.getEditorState();
        this.onChange(RichUtils.toggleBlockType(editorState, blockType));
    }
    toggleInlineStyle(inlineStyle) {
        const editorState = this.getEditorState();
        this.onChange(RichUtils.toggleInlineStyle(editorState, inlineStyle));
    }
    addHR() {
        const editorState = this.getEditorState();
        this.onChange(DraftUtils.addHorizontalRuleRemovingSelection(editorState));
    }
    addBR() {
        const editorState = this.getEditorState();
        this.onChange(DraftUtils.addLineBreak(editorState));
    }
    blockRenderer(block) {
        const { entityTypes, textDirectionality, placeholder, blockTypes } = this.props;
        const editorState = this.getEditorState();
        const contentState = editorState.getCurrentContent();
        if (block.getType() !== BLOCK_TYPE.ATOMIC) {
            // Render an (editable) custom placeholder block for empty non-atomic blocks.
            if (block.getText() === "") {
                return {
                    component: DraftailPlaceholderBlock,
                    props: {
                        placeholder,
                        blockTypes,
                    },
                };
            }
            return null;
        }
        const entityKey = block.getEntityAt(0);
        if (!entityKey) {
            return {
                editable: false,
            };
        }
        const entity = contentState.getEntity(entityKey);
        const isHorizontalRule = entity.getType() === ENTITY_TYPE.HORIZONTAL_RULE;
        if (isHorizontalRule) {
            return {
                component: DividerBlock,
                editable: false,
            };
        }
        const entityType = entityTypes.find((t) => t.type === entity.getType());
        return {
            component: entityType.block,
            editable: false,
            props: {
                /** The editorState is available for arbitrary content manipulation. */
                editorState,
                /** Current entity to manage. */
                entity,
                /** Current entityKey to manage. */
                entityKey,
                /** Whole entityType configuration, as provided to the editor. */
                entityType,
                /** Optionally set the overriding text directionality for this editor. */
                textDirectionality,
                /** Make the whole editor read-only, except for the block. */
                lockEditor: this.lockEditor,
                /** Make the editor editable again. */
                unlockEditor: this.unlockEditor,
                /** Shorthand to edit entity data. */
                onEditEntity: this.onEditEntity.bind(null, entityKey),
                /** Shorthand to remove an entity, and the related block. */
                onRemoveEntity: this.onRemoveEntity.bind(null, entityKey, block.getKey()),
                /** Update the editorState with arbitrary changes. */
                onChange: this.onChange,
            },
        };
    }
    /**
     * Imperative focus API similar to that of Draft.js.
     * See https://draftjs.org/docs/advanced-topics-managing-focus.html#content.
     */
    focus() {
        this.editorRef.focus();
    }
    renderSource() {
        const { textDirectionality } = this.props;
        const { sourceOptions } = this.state;
        const editorState = this.getEditorState();
        if (sourceOptions && sourceOptions.entityType) {
            const Source = sourceOptions.entityType.source;
            return (React.createElement(Source
            /** The editorState is available for arbitrary content manipulation. */
            , { 
                /** The editorState is available for arbitrary content manipulation. */
                editorState: editorState, 
                /** Takes the updated editorState, or null if there are no changes, and focuses the editor. */
                onComplete: this.onCompleteSource, 
                /** Closes the source, without focusing the editor again. */
                onClose: this.onCloseSource, 
                /** Current entity to edit, if any. */
                entity: sourceOptions.entity, 
                /** Current entityKey to edit, if any. */
                entityKey: sourceOptions.entityKey, 
                /** Whole entityType configuration, as provided to the editor. */
                entityType: sourceOptions.entityType, 
                /** Optionally set the overriding text directionality for this editor. */
                textDirectionality: textDirectionality }));
        }
        return null;
    }
    renderCommandPalette() {
        const { textDirectionality, blockTypes } = this.props;
        return (React.createElement(CommandPalette, { textDirectionality: textDirectionality, blockTypes: blockTypes, getEditorState: this.getEditorState, onCompleteSource: this.onCompleteSource }));
    }
    render() {
        const { placeholder, enableHorizontalRule, enableLineBreak, showUndoControl, showRedoControl, stripPastedStyles, multiline, spellCheck, textAlignment, textDirectionality, autoCapitalize, autoComplete, autoCorrect, ariaDescribedBy, ariaExpanded, ariaLabel, ariaLabelledBy, ariaOwneeID, ariaRequired, blockTypes, inlineStyles, entityTypes, decorators, controls, readOnly, maxListNesting, plugins, topToolbar, bottomToolbar, } = this.props;
        const { hasFocus, readOnlyState } = this.state;
        const editorState = this.getEditorState();
        const isReadOnly = readOnlyState || readOnly;
        const hidePlaceholder = DraftUtils.shouldHidePlaceholder(editorState);
        const entityDecorators = entityTypes
            .filter((type) => !!type.decorator)
            .map((type) => ({
            strategy: DraftUtils.getEntityTypeStrategy(type.type),
            component: decorateComponentWithProps(type.decorator, {
                onEdit: this.onEditEntity,
                onRemove: this.onRemoveEntity,
                textDirectionality,
            }),
        }));
        const TopToolbar = topToolbar;
        const BottomToolbar = bottomToolbar;
        const selectedBlock = DraftUtils.getSelectedBlock(editorState);
        const toolbarProps = {
            currentStyles: editorState.getCurrentInlineStyle(),
            currentBlock: selectedBlock.getType(),
            currentBlockKey: selectedBlock.getKey(),
            enableHorizontalRule,
            enableLineBreak,
            showUndoControl,
            showRedoControl,
            blockTypes,
            inlineStyles,
            entityTypes,
            controls,
            readOnly: isReadOnly,
            toggleBlockType: this.toggleBlockType,
            toggleInlineStyle: this.toggleInlineStyle,
            addHR: this.addHR,
            addBR: this.addBR,
            onUndoRedo: this.onUndoRedo,
            onRequestSource: this.onRequestSource,
            onCompleteSource: this.onCompleteSource,
            getEditorState: this.getEditorState,
            focus: this.focus,
            onChange: this.onChange,
        };
        return (React.createElement("div", { className: `Draftail-Editor${isReadOnly ? " Draftail-Editor--readonly" : ""}${hidePlaceholder ? " Draftail-Editor--hide-placeholder" : ""}${hasFocus ? " Draftail-Editor--focus" : ""}`, dir: textDirectionality === "RTL" ? "rtl" : "ltr", "data-draftail-editor": true },
            TopToolbar ? React.createElement(TopToolbar, { ...toolbarProps }) : null,
            React.createElement("div", { className: "Draftail-Editor__wrapper" },
                React.createElement(Editor, { customStyleMap: behavior.getCustomStyleMap(inlineStyles), ref: (ref) => {
                        this.editorRef = ref;
                    }, editorState: editorState, onChange: this.onChange, placeholder: placeholder, readOnly: isReadOnly, stripPastedStyles: stripPastedStyles, spellCheck: spellCheck, textAlignment: textAlignment, textDirectionality: textDirectionality, autoCapitalize: autoCapitalize, autoComplete: autoComplete, autoCorrect: autoCorrect, ariaDescribedBy: ariaDescribedBy, ariaExpanded: ariaExpanded, ariaLabel: ariaLabel, ariaLabelledBy: ariaLabelledBy, ariaMultiline: multiline, ariaOwneeID: ariaOwneeID, ariaRequired: ariaRequired, handleReturn: this.handleReturn, defaultKeyBindings: false, handleKeyCommand: this.handleKeyCommand, handleBeforeInput: this.handleBeforeInput, handlePastedText: this.handlePastedText, onFocus: this.onFocus, onBlur: this.onBlur, onTab: this.onTab, onUpArrow: this.onUpArrow, onDownArrow: this.onDownArrow, blockRendererFn: this.blockRenderer, blockRenderMap: behavior.getBlockRenderMap(blockTypes), blockStyleFn: behavior.blockStyleFn, 
                    // Include the keyBindingFn in a plugin here so that
                    // other plugin keyBindingFn's are still called, while
                    // still being able to override the Draft.js oversensitive
                    // keyboard shortcuts.
                    plugins: plugins.concat([
                        {
                            keyBindingFn: behavior.getKeyBindingFn(blockTypes, inlineStyles, entityTypes),
                        },
                    ]), decorators: decorators.concat(entityDecorators) })),
            BottomToolbar ? React.createElement(BottomToolbar, { ...toolbarProps }) : null,
            this.renderSource(),
            this.renderCommandPalette(),
            React.createElement(PlaceholderStyles, { blockKey: selectedBlock.getKey(), placeholder: placeholder }),
            React.createElement(ListNestingStyles, { max: maxListNesting })));
    }
}
DraftailEditor.defaultProps = defaultProps;

const showButton = (config) => Boolean(config.icon) || Boolean(getControlLabel(config.type, config));
const showEntityButton = (config) => showButton(config) && Boolean(config.decorator);
const getButtonTitle = (type, config) => {
    const description = typeof config === "boolean" || typeof config.description === "undefined"
        ? DESCRIPTIONS[type]
        : config.description;
    const hasShortcut = behavior.hasKeyboardShortcut(type);
    let title = description;
    if (hasShortcut) {
        const desc = description ? `${description}\n` : "";
        title = `${desc}${behavior.getKeyboardShortcut(type)}`;
    }
    return title;
};
class ToolbarDefaults extends PureComponent {
    render() {
        const { currentStyles, currentBlock, blockTypes, inlineStyles, entityTypes, toggleBlockType, toggleInlineStyle, onRequestSource, } = this.props;
        return [
            React.createElement(ToolbarGroup, { key: "styles" }, inlineStyles.filter(showButton).map((t) => (React.createElement(ToolbarButton, { key: t.type, name: t.type, active: currentStyles.has(t.type), label: getControlLabel(t.type, t), title: getButtonTitle(t.type, t), icon: t.icon, onClick: toggleInlineStyle })))),
            React.createElement(ToolbarGroup, { key: "blocks" }, blockTypes.filter(showButton).map((t) => (React.createElement(ToolbarButton, { key: t.type, name: t.type, active: currentBlock === t.type, label: getControlLabel(t.type, t), title: getButtonTitle(t.type, t), icon: t.icon, onClick: toggleBlockType })))),
            React.createElement(ToolbarGroup, { key: "entities" }, entityTypes.filter(showEntityButton).map((t) => (React.createElement(ToolbarButton, { key: t.type, name: t.type, onClick: onRequestSource, label: getControlLabel(t.type, t), title: getButtonTitle(t.type, t), icon: t.icon })))),
        ];
    }
}

const getReferenceClientRect = () => getVisibleSelectionRect(window);
const InlineToolbar = (props) => {
    // Support the legacy and current controls APIs.
    const { controls, getEditorState, onChange } = props;
    const tippyParentRef = useRef(null);
    const [selectionRect, setSelectionRect] = useState();
    const editorState = getEditorState();
    const selection = editorState.getSelection();
    const isCollapsed = selection.getHasFocus() && !selection.isCollapsed();
    useEffect(() => {
        if (isCollapsed) {
            setSelectionRect(getReferenceClientRect());
        }
        else {
            setSelectionRect(null);
        }
    }, [isCollapsed]);
    const isVisible = isCollapsed && Boolean(selectionRect);
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { ref: tippyParentRef }),
        isVisible ? (React.createElement(Tippy, { visible: isVisible, getReferenceClientRect: () => selectionRect, maxWidth: "100%", interactive: true, appendTo: () => tippyParentRef.current, content: React.createElement("div", { className: "Draftail-Toolbar", role: "toolbar" },
                React.createElement(ToolbarDefaults, { ...props }),
                React.createElement(ToolbarGroup, null, controls
                    .filter((control) => Boolean(control.inline))
                    .map((control, i) => {
                    const Control = control.inline;
                    return (React.createElement(Control
                    // eslint-disable-next-line @thibaudcolas/cookbook/react/no-array-index-key
                    , { 
                        // eslint-disable-next-line @thibaudcolas/cookbook/react/no-array-index-key
                        key: i, getEditorState: getEditorState, onChange: onChange }));
                }))) })) : null));
};

const addIcon = (React.createElement("svg", { width: "16", height: "16", viewBox: "0 0 448 512", "aria-hidden": "true" },
    React.createElement("path", { d: "M432 256c0 17.69-14.33 32.01-32 32.01H256v144c0 17.69-14.33 31.99-32 31.99s-32-14.3-32-31.99v-144H48c-17.67 0-32-14.32-32-32.01s14.33-31.99 32-31.99h144v-144C192 62.32 206.33 48 224 48s32 14.32 32 32.01v144h144c17.7-.01 32 14.29 32 31.99z" })));
const hideTooltipOnEsc = {
    name: "hideOnEsc",
    defaultValue: true,
    fn({ hide }) {
        function onKeyDown(e) {
            if (e.key === "Escape") {
                hide();
            }
        }
        return {
            onShow() {
                document.addEventListener("keydown", onKeyDown);
            },
            onHide() {
                document.removeEventListener("keydown", onKeyDown);
            },
        };
    },
};
const tippyPlugins = [hideTooltipOnEsc];
const BlockToolbar = ({ trigger, comboBox, controls, getEditorState, focus, onChange, blockTypes, currentBlock, currentBlockKey, toggleBlockType, onRequestSource, onCompleteSource, entityTypes, addHR, enableHorizontalRule, }) => {
    const tippyParentRef = useRef(null);
    const [selectionRect, setSelectionRect] = useState(null);
    const [visible, setVisible] = useState(false);
    const [toggleVisible, setToggleVisible] = useState(false);
    useState(false);
    const [selectedItem, setSelectedItem] = useState(null);
    const editorState = getEditorState();
    const selection = editorState.getSelection();
    const isCollapsed = selection.isCollapsed();
    const isStart = selection.getAnchorOffset() === 0;
    const anchorKey = selection.getAnchorKey();
    const showToggle = isCollapsed &&
        isStart &&
        DraftUtils.getSelectedBlock(editorState).getText() === "";
    useEffect(() => {
        if (showToggle) {
            const elt = document.querySelector(`[data-block="true"][data-offset-key="${anchorKey}-0-0"]`);
            setSelectionRect(elt.getBoundingClientRect());
            setToggleVisible(true);
        }
        else {
            setToggleVisible(false);
        }
        return () => {
            setToggleVisible(false);
        };
    }, [showToggle, anchorKey]);
    const items = [
        ...blockTypes.filter(showButton$1).map((t) => ({
            ...t,
            onSelect: () => {
                return RichUtils.toggleBlockType(getEditorState(), t.type);
            },
        })),
        ...entityTypes
            .filter(showButton$1)
            .filter((t) => Boolean(t.block))
            .map((t) => ({
            ...t,
            onSelect: onRequestSource.bind(null, t.type),
        })),
    ];
    if (enableHorizontalRule) {
        items.push({
            type: ENTITY_TYPE.HORIZONTAL_RULE,
            onSelect: addHR,
            ...(typeof enableHorizontalRule === "object" ? enableHorizontalRule : {}),
        });
    }
    return (React.createElement("div", { className: `Draftail-BlockToolbar Draftail-BlockToolbar--${visible ? "open" : "closed"}` },
        React.createElement("div", { ref: tippyParentRef }),
        React.createElement(Tippy, { maxWidth: "100%", interactive: true, visible: visible, onClickOutside: () => setVisible(false), trigger: "click", placement: comboBox.placement, arrow: false, appendTo: () => tippyParentRef.current, onMount: (instance) => {
                const field = instance.popper.querySelector('[aria-autocomplete="list"]');
                if (field) {
                    field.focus();
                }
            }, onHidden: () => {
                if (!showToggle) {
                    setToggleVisible(false);
                }
            }, plugins: tippyPlugins, content: React.createElement(ComboBox, { key: `${currentBlockKey}-${currentBlock}`, label: comboBox.label, placeholder: comboBox.placeholder, selectedItem: selectedItem, items: items, onSelect: (selection) => {
                    setSelectedItem(selection.selectedItem);
                    setVisible(false);
                    onCompleteSource(selection.selectedItem.onSelect());
                } }) },
            React.createElement("button", { type: "button", "aria-expanded": toggleVisible ? "true" : "false", className: "Draftail-BlockToolbar__trigger", style: {
                    top: selectionRect ? selectionRect.top + window.scrollY : 0,
                    visibility: toggleVisible ? "visible" : "hidden",
                }, "aria-label": trigger.label, onClick: () => setVisible(true) }, trigger.icon)),
        React.createElement("div", { className: "Draftail-BlockToolbar__backdrop" })));
};
BlockToolbar.defaultProps = {
    trigger: {
        icon: addIcon,
        label: "Insert block",
    },
    comboBox: {
        label: "Choose an item:",
        placeholder: "Search blocks",
        placement: "auto-end",
    },
};

const MetaToolbar = ({ showBlockEntities, entityTypes, controls, getEditorState, onChange, onRequestSource, }) => (React.createElement("div", { className: "Draftail-MetaToolbar" },
    showBlockEntities ? (React.createElement(ToolbarGroup, { key: "entities" }, entityTypes
        .filter((entityType) => showButton$1(entityType) && entityType.block)
        .map((t) => (React.createElement(ToolbarButton, { key: t.type, name: t.type, onClick: onRequestSource, label: getControlLabel(t.type, t), title: getButtonTitle$1(t.type, t), icon: t.icon }))))) : null,
    React.createElement(ToolbarGroup, null, controls.map((control, i) => {
        if (control.inline || control.block) {
            return null;
        }
        // Support the legacy and current controls APIs.
        const Control = control.meta || control;
        return (React.createElement(Control
        // eslint-disable-next-line @thibaudcolas/cookbook/react/no-array-index-key
        , { 
            // eslint-disable-next-line @thibaudcolas/cookbook/react/no-array-index-key
            key: i, getEditorState: getEditorState, onChange: onChange }));
    }))));

export { BLOCK_TYPE, BlockToolbar, DraftUtils, DraftailEditor, ENTITY_TYPE, INLINE_STYLE, Icon, InlineToolbar, MetaToolbar, Toolbar, ToolbarButton };
